@@ -4,12 +4,13 @@
 import csv
 import importlib
 from collections import defaultdict
-
-from pyhocon import ConfigTree
 from typing import Any
 
+from pyhocon import ConfigTree
+
 from databuilder.extractor.base_extractor import Extractor
-from databuilder.models.table_metadata import TableMetadata, ColumnMetadata
+from databuilder.models.badge import Badge, BadgeMetadata
+from databuilder.models.table_metadata import ColumnMetadata, TableMetadata
 
 
 class CsvExtractor(Extractor):
@@ -65,6 +66,83 @@ class CsvExtractor(Extractor):
         return 'extractor.csv'
 
 
+class CsvTableBadgeExtractor(Extractor):
+    # Config keys
+    TABLE_FILE_LOCATION = 'table_file_location'
+    BADGE_FILE_LOCATION = 'badge_file_location'
+
+    """
+    An Extractor that combines Table and Badge CSVs.
+    """
+    def init(self, conf: ConfigTree) -> None:
+        self.conf = conf
+        self.table_file_location = conf.get_string(CsvTableBadgeExtractor.TABLE_FILE_LOCATION)
+        self.badge_file_location = conf.get_string(CsvTableBadgeExtractor.BADGE_FILE_LOCATION)
+        self._load_csv()
+
+    def _get_key(self,
+                 db: str,
+                 cluster: str,
+                 schema: str,
+                 tbl: str
+                 ) -> str:
+        return TableMetadata.TABLE_KEY_FORMAT.format(db=db,
+                                                     cluster=cluster,
+                                                     schema=schema,
+                                                     tbl=tbl)
+
+    def _load_csv(self) -> None:
+        with open(self.badge_file_location, 'r') as fin:
+            self.badges = [dict(i) for i in csv.DictReader(fin)]
+        # print("BADGES: " + str(self.badges))
+
+        parsed_badges = defaultdict(list)
+        for badge_dict in self.badges:
+            db = badge_dict['database']
+            cluster = badge_dict['cluster']
+            schema = badge_dict['schema']
+            table_name = badge_dict['table_name']
+            id = self._get_key(db, cluster, schema, table_name)
+            badge = Badge(name=badge_dict['name'],
+                          category=badge_dict['category'])
+            parsed_badges[id].append(badge)
+
+        with open(self.table_file_location, 'r') as fin:
+            tables = [dict(i) for i in csv.DictReader(fin)]
+
+        results = []
+        for table_dict in tables:
+            db = table_dict['database']
+            cluster = table_dict['cluster']
+            schema = table_dict['schema']
+            table_name = table_dict['name']
+            id = self._get_key(db, cluster, schema, table_name)
+            badges = parsed_badges[id]
+
+            if badges is None:
+                badges = []
+            badge_metadata = BadgeMetadata(start_label=TableMetadata.TABLE_NODE_LABEL,
+                                           start_key=id,
+                                           badges=badges)
+            results.append(badge_metadata)
+        self._iter = iter(results)
+
+    def extract(self) -> Any:
+        """
+        Yield the csv result one at a time.
+        convert the result to model if a model_class is provided
+        """
+        try:
+            return next(self._iter)
+        except StopIteration:
+            return None
+        except Exception as e:
+            raise e
+
+    def get_scope(self) -> str:
+        return 'extractor.csvtablebadge'
+
+
 class CsvTableColumnExtractor(Extractor):
     # Config keys
     TABLE_FILE_LOCATION = 'table_file_location'
@@ -113,7 +191,8 @@ class CsvTableColumnExtractor(Extractor):
                 name=column_dict['name'],
                 description=column_dict['description'],
                 col_type=column_dict['col_type'],
-                sort_order=int(column_dict['sort_order'])
+                sort_order=int(column_dict['sort_order']),
+                badges=[column_dict['badges']]
             )
             parsed_columns[id].append(column)
 
