@@ -1,16 +1,19 @@
 # Copyright Contributors to the Amundsen project.
 # SPDX-License-Identifier: Apache-2.0
 
-from pyhocon import ConfigTree, ConfigFactory
-from requests.auth import HTTPBasicAuth
 from typing import Any, Dict
 
+from pyhocon import ConfigFactory, ConfigTree
+from requests.auth import HTTPBasicAuth
+
 from databuilder import Scoped
-from databuilder.extractor.dashboard.mode_analytics.mode_dashboard_constants import ORGANIZATION, MODE_ACCESS_TOKEN, \
-    MODE_PASSWORD_TOKEN, MODE_BEARER_TOKEN
-from databuilder.extractor.restapi.rest_api_extractor import RestAPIExtractor, REST_API_QUERY, STATIC_RECORD_DICT
-from databuilder.rest_api.base_rest_api_query import BaseRestApiQuery
-from databuilder.rest_api.base_rest_api_query import RestApiQuerySeed
+from databuilder.extractor.dashboard.mode_analytics.mode_dashboard_constants import (
+    MODE_ACCESS_TOKEN, MODE_BEARER_TOKEN, MODE_PASSWORD_TOKEN, ORGANIZATION,
+)
+from databuilder.extractor.restapi.rest_api_extractor import (
+    REST_API_QUERY, STATIC_RECORD_DICT, RestAPIExtractor,
+)
+from databuilder.rest_api.base_rest_api_query import BaseRestApiQuery, RestApiQuerySeed
 from databuilder.rest_api.rest_api_query import RestApiQuery
 
 
