@@ -1,28 +1,25 @@
 # Copyright Contributors to the Amundsen project.
 # SPDX-License-Identifier: Apache-2.0
 
-import copy
 import csv
 import ctypes
-from io import open
 import logging
 import time
+from io import open
 from os import listdir
 from os.path import isfile, join
-from string import Template
+from typing import List, Set
 
-import six
-from neo4j import GraphDatabase, Transaction  # noqa: F401
 import neo4j
-from neo4j.exceptions import CypherError
-from pyhocon import ConfigFactory  # noqa: F401
-from pyhocon import ConfigTree  # noqa: F401
-from typing import Set, List  # noqa: F401
+import pandas
+from jinja2 import Template
+from neo4j import GraphDatabase, Transaction
+from neo4j.exceptions import CypherError, TransientError
+from pyhocon import ConfigFactory, ConfigTree
 
 from databuilder.publisher.base_publisher import Publisher
 from databuilder.publisher.neo4j_preprocessor import NoopRelationPreprocessor
 
-
 # Setting field_size_limit to solve the error below
 # _csv.Error: field larger than field limit (131072)
 # https://stackoverflow.com/a/54517228/5972935
@@ -36,7 +33,7 @@ RELATION_FILES_DIR = 'relation_files_directory'
 # A end point for Neo4j e.g: bolt://localhost:9999
 NEO4J_END_POINT_KEY = 'neo4j_endpoint'
 # A transaction size that determines how often it commits.
-NEO4J_TRANSCATION_SIZE = 'neo4j_transaction_size'
+NEO4J_TRANSACTION_SIZE = 'neo4j_transaction_size'
 # A progress report frequency that determines how often it report the progress.
 NEO4J_PROGRESS_REPORT_FREQUENCY = 'neo4j_progress_report_frequency'
 # A boolean flag to make it fail if relationship is not created
@@ -47,6 +44,9 @@ NEO4J_MAX_CONN_LIFE_TIME_SEC = 'neo4j_max_conn_life_time_sec'
 # list of nodes that are create only, and not updated if match exists
 NEO4J_CREATE_ONLY_NODES = 'neo4j_create_only_nodes'
 
+# list of node labels that could attempt to be accessed simultaneously
+NEO4J_DEADLOCK_NODE_LABELS = 'neo4j_deadlock_node_labels'
+
 NEO4J_USER = 'neo4j_user'
 NEO4J_PASSWORD = 'neo4j_password'
 NEO4J_ENCRYPTED = 'neo4j_encrypted'
@@ -93,7 +93,7 @@ RELATION_REQUIRED_KEYS = {RELATION_START_LABEL, RELATION_START_KEY,
                           RELATION_END_LABEL, RELATION_END_KEY,
                           RELATION_TYPE, RELATION_REVERSE_TYPE}
 
-DEFAULT_CONFIG = ConfigFactory.from_dict({NEO4J_TRANSCATION_SIZE: 500,
+DEFAULT_CONFIG = ConfigFactory.from_dict({NEO4J_TRANSACTION_SIZE: 500,
                                           NEO4J_PROGRESS_REPORT_FREQUENCY: 500,
                                           NEO4J_RELATIONSHIP_CREATION_CONFIRM: False,
                                           NEO4J_MAX_CONN_LIFE_TIME_SEC: 50,
@@ -101,18 +101,9 @@ DEFAULT_CONFIG = ConfigFactory.from_dict({NEO4J_TRANSCATION_SIZE: 500,
                                           NEO4J_VALIDATE_SSL: False,
                                           RELATION_PREPROCESSOR: NoopRelationPreprocessor()})
 
-NODE_MERGE_TEMPLATE = Template("""MERGE (node:$LABEL {key: '${KEY}'})
-ON CREATE SET ${create_prop_body}
-${update_statement}""")
-
-NODE_UPDATE_TEMPLATE = Template("""ON MATCH SET ${update_prop_body}""")
-
-RELATION_MERGE_TEMPLATE = Template("""MATCH (n1:$START_LABEL {key: '${START_KEY}'}),
-(n2:$END_LABEL {key: '${END_KEY}'})
-MERGE (n1)-[r1:$TYPE]->(n2)-[r2:$REVERSE_TYPE]->(n1)
-$PROP_STMT RETURN n1.key, n2.key""")
-
-CREATE_UNIQUE_INDEX_TEMPLATE = Template('CREATE CONSTRAINT ON (node:${LABEL}) ASSERT node.key IS UNIQUE')
+# transient error retries and sleep time
+RETRIES_NUMBER = 5
+SLEEP_TIME = 2
 
 LOGGER = logging.getLogger(__name__)
 
@@ -128,15 +119,13 @@ class Neo4jCsvPublisher(Publisher):
     #TODO User UNWIND batch operation for better performance
     """
 
-    def __init__(self):
-        # type: () -> None
+    def __init__(self) -> None:
         super(Neo4jCsvPublisher, self).__init__()
 
-    def init(self, conf):
-        # type: (ConfigTree) -> None
+    def init(self, conf: ConfigTree) -> None:
         conf = conf.with_fallback(DEFAULT_CONFIG)
 
-        self._count = 0  # type: int
+        self._count: int = 0
         self._progress_report_frequency = conf.get_int(NEO4J_PROGRESS_REPORT_FREQUENCY)
         self._node_files = self._list_files(conf, NODE_FILES_DIR)
         self._node_files_iter = iter(self._node_files)
@@ -152,25 +141,24 @@ class Neo4jCsvPublisher(Publisher):
                                  auth=(conf.get_string(NEO4J_USER), conf.get_string(NEO4J_PASSWORD)),
                                  encrypted=conf.get_bool(NEO4J_ENCRYPTED),
                                  trust=trust)
-        self._transaction_size = conf.get_int(NEO4J_TRANSCATION_SIZE)
+        self._transaction_size = conf.get_int(NEO4J_TRANSACTION_SIZE)
         self._session = self._driver.session()
         self._confirm_rel_created = conf.get_bool(NEO4J_RELATIONSHIP_CREATION_CONFIRM)
 
         # config is list of node label.
         # When set, this list specifies a list of nodes that shouldn't be updated, if exists
         self.create_only_nodes = set(conf.get_list(NEO4J_CREATE_ONLY_NODES, default=[]))
-        self.labels = set()  # type: Set[str]
-        self.publish_tag = conf.get_string(JOB_PUBLISH_TAG)  # type: str
+        self.deadlock_node_labels = set(conf.get_list(NEO4J_DEADLOCK_NODE_LABELS, default=[]))
+        self.labels: Set[str] = set()
+        self.publish_tag: str = conf.get_string(JOB_PUBLISH_TAG)
         if not self.publish_tag:
-            raise Exception('{} should not be empty'.format(JOB_PUBLISH_TAG))
+            raise Exception(f'{JOB_PUBLISH_TAG} should not be empty')
 
         self._relation_preprocessor = conf.get(RELATION_PREPROCESSOR)
 
-        LOGGER.info('Publishing Node csv files {}, and Relation CSV files {}'
-                    .format(self._node_files, self._relation_files))
+        LOGGER.info('Publishing Node csv files %s, and Relation CSV files %s', self._node_files, self._relation_files)
 
-    def _list_files(self, conf, path_key):
-        # type: (ConfigTree, str) -> List[str]
+    def _list_files(self, conf: ConfigTree, path_key: str) -> List[str]:
         """
         List files from directory
         :param conf:
@@ -183,8 +171,7 @@ class Neo4jCsvPublisher(Publisher):
         path = conf.get_string(path_key)
         return [join(path, f) for f in listdir(path) if isfile(join(path, f))]
 
-    def publish_impl(self):  # noqa: C901
-        # type: () -> None
+    def publish_impl(self) -> None:  # noqa: C901
         """
         Publishes Nodes first and then Relations
         :return:
@@ -192,11 +179,11 @@ class Neo4jCsvPublisher(Publisher):
 
         start = time.time()
 
-        LOGGER.info('Creating indices using Node files: {}'.format(self._node_files))
+        LOGGER.info('Creating indices using Node files: %s', self._node_files)
         for node_file in self._node_files:
             self._create_indices(node_file=node_file)
 
-        LOGGER.info('Publishing Node files: {}'.format(self._node_files))
+        LOGGER.info('Publishing Node files: %s', self._node_files)
         try:
             tx = self._session.begin_transaction()
             while True:
@@ -206,7 +193,7 @@ class Neo4jCsvPublisher(Publisher):
                 except StopIteration:
                     break
 
-            LOGGER.info('Publishing Relationship files: {}'.format(self._relation_files))
+            LOGGER.info('Publishing Relationship files: %s', self._relation_files)
             while True:
                 try:
                     relation_file = next(self._relation_files_iter)
@@ -215,31 +202,29 @@ class Neo4jCsvPublisher(Publisher):
                     break
 
             tx.commit()
-            LOGGER.info('Committed total {} statements'.format(self._count))
+            LOGGER.info('Committed total %i statements', self._count)
 
             # TODO: Add statsd support
-            LOGGER.info('Successfully published. Elapsed: {} seconds'.format(time.time() - start))
+            LOGGER.info('Successfully published. Elapsed: %i seconds', time.time() - start)
         except Exception as e:
             LOGGER.exception('Failed to publish. Rolling back.')
             if not tx.closed():
                 tx.rollback()
             raise e
 
-    def get_scope(self):
-        # type: () -> str
+    def get_scope(self) -> str:
         return 'publisher.neo4j'
 
-    def _create_indices(self, node_file):
+    def _create_indices(self, node_file: str) -> None:
         """
         Go over the node file and try creating unique index
         :param node_file:
         :return:
         """
-        # type: (str) -> None
         LOGGER.info('Creating indices. (Existing indices will be ignored)')
 
         with open(node_file, 'r', encoding='utf8') as node_csv:
-            for node_record in csv.DictReader(node_csv):
+            for node_record in pandas.read_csv(node_csv, na_filter=False).to_dict(orient='records'):
                 label = node_record[NODE_LABEL_KEY]
                 if label not in self.labels:
                     self._try_create_index(label)
@@ -247,8 +232,7 @@ class Neo4jCsvPublisher(Publisher):
 
         LOGGER.info('Indices have been created.')
 
-    def _publish_node(self, node_file, tx):
-        # type: (str, Transaction) -> Transaction
+    def _publish_node(self, node_file: str, tx: Transaction) -> Transaction:
         """
         Iterate over the csv records of a file, each csv record transform to Merge statement and will be executed.
         All nodes should have a unique key, and this method will try to create unique index on the LABEL when it sees
@@ -267,13 +251,13 @@ class Neo4jCsvPublisher(Publisher):
         """
 
         with open(node_file, 'r', encoding='utf8') as node_csv:
-            for count, node_record in enumerate(csv.DictReader(node_csv)):
+            for node_record in pandas.read_csv(node_csv, na_filter=False).to_dict(orient="records"):
                 stmt = self.create_node_merge_statement(node_record=node_record)
-                tx = self._execute_statement(stmt, tx)
+                params = self._create_props_param(node_record)
+                tx = self._execute_statement(stmt, tx, params)
         return tx
 
-    def is_create_only_node(self, node_record):
-        # type: (dict) -> bool
+    def is_create_only_node(self, node_record: dict) -> bool:
         """
         Check if node can be updated
         :param node_record:
@@ -284,26 +268,25 @@ class Neo4jCsvPublisher(Publisher):
         else:
             return False
 
-    def create_node_merge_statement(self, node_record):
-        # type: (dict) -> str
+    def create_node_merge_statement(self, node_record: dict) -> str:
         """
         Creates node merge statement
         :param node_record:
         :return:
         """
-        params = copy.deepcopy(node_record)
-        params['create_prop_body'] = self._create_props_body(node_record, NODE_REQUIRED_KEYS, 'node')
+        template = Template("""
+            MERGE (node:{{ LABEL }} {key: $KEY})
+            ON CREATE SET {{ PROP_BODY }}
+            {% if update %} ON MATCH SET {{ PROP_BODY }} {% endif %}
+        """)
 
-        update_statement = ''
-        if not self.is_create_only_node(node_record):
-            update_prop_body = self._create_props_body(node_record, NODE_REQUIRED_KEYS, 'node')
-            update_statement = NODE_UPDATE_TEMPLATE.substitute(update_prop_body=update_prop_body)
-        params['update_statement'] = update_statement
+        prop_body = self._create_props_body(node_record, NODE_REQUIRED_KEYS, 'node')
 
-        return NODE_MERGE_TEMPLATE.substitute(params)
+        return template.render(LABEL=node_record["LABEL"],
+                               PROP_BODY=prop_body,
+                               update=(not self.is_create_only_node(node_record)))
 
-    def _publish_relation(self, relation_file, tx):
-        # type: (str, Transaction) -> Transaction
+    def _publish_relation(self, relation_file: str, tx: Transaction) -> Transaction:
         """
         Creates relation between two nodes.
         (In Amundsen, all relation is bi-directional)
@@ -319,11 +302,12 @@ class Neo4jCsvPublisher(Publisher):
         """
 
         if self._relation_preprocessor.is_perform_preprocess():
-            LOGGER.info('Pre-processing relation with {}'.format(self._relation_preprocessor))
+            LOGGER.info('Pre-processing relation with %s', self._relation_preprocessor)
 
             count = 0
             with open(relation_file, 'r', encoding='utf8') as relation_csv:
-                for rel_record in csv.DictReader(relation_csv):
+                for rel_record in pandas.read_csv(relation_csv, na_filter=False).to_dict(orient="records"):
+                    # TODO not sure if deadlock on badge node arises in preporcessing or not
                     stmt, params = self._relation_preprocessor.preprocess_cypher(
                         start_label=rel_record[RELATION_START_LABEL],
                         end_label=rel_record[RELATION_END_LABEL],
@@ -336,45 +320,69 @@ class Neo4jCsvPublisher(Publisher):
                         tx = self._execute_statement(stmt, tx=tx, params=params)
                         count += 1
 
-            LOGGER.info('Executed pre-processing Cypher statement {} times'.format(count))
+            LOGGER.info('Executed pre-processing Cypher statement %i times', count)
 
         with open(relation_file, 'r', encoding='utf8') as relation_csv:
-            for count, rel_record in enumerate(csv.DictReader(relation_csv)):
-                stmt = self.create_relationship_merge_statement(rel_record=rel_record)
-                tx = self._execute_statement(stmt, tx,
-                                             expect_result=self._confirm_rel_created)
+            for rel_record in pandas.read_csv(relation_csv, na_filter=False).to_dict(orient="records"):
+                exception_exists = True
+                retries_for_exception = RETRIES_NUMBER
+                while exception_exists and retries_for_exception > 0:
+                    try:
+                        stmt = self.create_relationship_merge_statement(rel_record=rel_record)
+                        params = self._create_props_param(rel_record)
+                        tx = self._execute_statement(stmt, tx, params,
+                                                     expect_result=self._confirm_rel_created)
+                        exception_exists = False
+                    except TransientError as e:
+                        if rel_record[RELATION_START_LABEL] in self.deadlock_node_labels \
+                                or rel_record[RELATION_END_LABEL] in self.deadlock_node_labels:
+                            time.sleep(SLEEP_TIME)
+                            retries_for_exception -= 1
+                        else:
+                            raise e
 
         return tx
 
-    def create_relationship_merge_statement(self, rel_record):
-        # type: (dict) -> str
+    def create_relationship_merge_statement(self, rel_record: dict) -> str:
         """
         Creates relationship merge statement
         :param rel_record:
         :return:
         """
-        param = copy.deepcopy(rel_record)
-        create_prop_body = self._create_props_body(rel_record, RELATION_REQUIRED_KEYS, 'r1')
-        param['PROP_STMT'] = ' '  # No properties for relationship by default
+        template = Template("""
+            MATCH (n1:{{ START_LABEL }} {key: $START_KEY}), (n2:{{ END_LABEL }} {key: $END_KEY})
+            MERGE (n1)-[r1:{{ TYPE }}]->(n2)-[r2:{{ REVERSE_TYPE }}]->(n1)
+            {% if update_prop_body %}
+            ON CREATE SET {{ prop_body }}
+            ON MATCH SET {{ prop_body }}
+            {% endif %}
+            RETURN n1.key, n2.key
+        """)
 
-        if create_prop_body:
-            # We need one more body for reverse relation
-            create_prop_body = ' , '.join([create_prop_body,
-                                           self._create_props_body(rel_record, RELATION_REQUIRED_KEYS, 'r2')])
-            update_prop_body = ' , '.join([self._create_props_body(rel_record, RELATION_REQUIRED_KEYS, 'r1'),
-                                           self._create_props_body(rel_record, RELATION_REQUIRED_KEYS, 'r2')])
+        prop_body_r1 = self._create_props_body(rel_record, RELATION_REQUIRED_KEYS, 'r1')
+        prop_body_r2 = self._create_props_body(rel_record, RELATION_REQUIRED_KEYS, 'r2')
+        prop_body = ' , '.join([prop_body_r1, prop_body_r2])
 
-            param['PROP_STMT'] = """ON CREATE SET {create_prop_body}
-ON MATCH SET {update_prop_body}""".format(create_prop_body=create_prop_body,
-                                          update_prop_body=update_prop_body)
+        return template.render(START_LABEL=rel_record["START_LABEL"],
+                               END_LABEL=rel_record["END_LABEL"],
+                               TYPE=rel_record["TYPE"],
+                               REVERSE_TYPE=rel_record["REVERSE_TYPE"],
+                               update_prop_body=prop_body_r1,
+                               prop_body=prop_body)
 
-        return RELATION_MERGE_TEMPLATE.substitute(param)
+    def _create_props_param(self, record_dict: dict) -> dict:
+        params = {}
+        for k, v in record_dict.items():
+            if k.endswith(UNQUOTED_SUFFIX):
+                k = k[:-len(UNQUOTED_SUFFIX)]
+
+            params[k] = v
+        return params
 
     def _create_props_body(self,
-                           record_dict,
-                           excludes,
-                           identifier):
-        # type: (dict, Set, str) -> str
+                           record_dict: dict,
+                           excludes: Set,
+                           identifier: str) -> str:
         """
         Creates properties body with params required for resolving template.
 
@@ -384,46 +392,28 @@ ON MATCH SET {update_prop_body}""".format(create_prop_body=create_prop_body,
         :param record_dict: A dict represents CSV row
         :param excludes: set of excluded columns that does not need to be in properties (e.g: KEY, LABEL ...)
         :param identifier: identifier that will be used in CYPHER query as shown on above example
-        :param is_update: Creates property body for update statement in MERGE
         :return: Properties body for Cypher statement
         """
-        template_params = {}
         props = []
-        for k, v in six.iteritems(record_dict):
+        for k, v in record_dict.items():
             if k in excludes:
-                template_params[k] = v
                 continue
 
-            # if isinstance(str, v):
-            # escape backslash for Cypher query
-            v = v.replace('\\', '\\\\')
-            # escape quote for Cypher query
-            v = v.replace('\'', "\\'")
-
             if k.endswith(UNQUOTED_SUFFIX):
                 k = k[:-len(UNQUOTED_SUFFIX)]
-                props.append('{id}.{key} = {val}'.format(id=identifier, key=k, val=v))
-            else:
-                props.append("""{id}.{key} = '{val}'""".format(id=identifier, key=k, val=v))
 
-            template_params[k] = v
+            props.append(f'{identifier}.{k} = ${k}')
 
-        props.append("""{id}.{key} = '{val}'""".format(id=identifier,
-                                                       key=PUBLISHED_TAG_PROPERTY_NAME,
-                                                       val=self.publish_tag))
-
-        props.append("""{id}.{key} = {val}""".format(id=identifier,
-                                                     key=LAST_UPDATED_EPOCH_MS,
-                                                     val='timestamp()'))
+        props.append(f"{identifier}.{PUBLISHED_TAG_PROPERTY_NAME} = '{self.publish_tag}'")
+        props.append(f"{identifier}.{LAST_UPDATED_EPOCH_MS} = timestamp()")
 
         return ', '.join(props)
 
     def _execute_statement(self,
-                           stmt,
-                           tx,
-                           params=None,
-                           expect_result=False):
-        # type: (str, Transaction, bool) -> Transaction
+                           stmt: str,
+                           tx: Transaction,
+                           params: dict = None,
+                           expect_result: bool = False) -> Transaction:
         """
         Executes statement against Neo4j. If execution fails, it rollsback and raise exception.
         If 'expect_result' flag is True, it confirms if result object is not null.
@@ -434,24 +424,20 @@ ON MATCH SET {update_prop_body}""".format(create_prop_body=create_prop_body,
         :return:
         """
         try:
-            if LOGGER.isEnabledFor(logging.DEBUG):
-                LOGGER.debug('Executing statement: {} with params {}'.format(stmt, params))
+            LOGGER.debug('Executing statement: %s with params %s', stmt, params)
 
-            if six.PY2:
-                result = tx.run(unicode(stmt, errors='ignore'), parameters=params)  # noqa
-            else:
-                result = tx.run(str(stmt).encode('utf-8', 'ignore'), parameters=params)
+            result = tx.run(str(stmt).encode('utf-8', 'ignore'), parameters=params)
             if expect_result and not result.single():
-                raise RuntimeError('Failed to executed statement: {}'.format(stmt))
+                raise RuntimeError(f'Failed to executed statement: {stmt}')
 
             self._count += 1
             if self._count > 1 and self._count % self._transaction_size == 0:
                 tx.commit()
-                LOGGER.info('Committed {} statements so far'.format(self._count))
+                LOGGER.info(f'Committed {self._count} statements so far')
                 return self._session.begin_transaction()
 
             if self._count > 1 and self._count % self._progress_report_frequency == 0:
-                LOGGER.info('Processed {} statements so far'.format(self._count))
+                LOGGER.info(f'Processed {self._count} statements so far')
 
             return tx
         except Exception as e:
@@ -460,18 +446,18 @@ ON MATCH SET {update_prop_body}""".format(create_prop_body=create_prop_body,
                 tx.rollback()
             raise e
 
-    def _try_create_index(self,
-                          label):
-        # type: (str) -> None
+    def _try_create_index(self, label: str) -> None:
         """
         For any label seen first time for this publisher it will try to create unique index.
         Neo4j ignores a second creation in 3.x, but raises an error in 4.x.
         :param label:
         :return:
         """
-        stmt = CREATE_UNIQUE_INDEX_TEMPLATE.substitute(LABEL=label)
-        LOGGER.info('Trying to create index for label {label} if not exist: {stmt}'.format(label=label,
-                                                                                           stmt=stmt))
+        stmt = Template("""
+            CREATE CONSTRAINT ON (node:{{ LABEL }}) ASSERT node.key IS UNIQUE
+        """).render(LABEL=label)
+
+        LOGGER.info(f'Trying to create index for label %s if not exist: %s', label, stmt)
         with self._driver.session() as session:
             try:
                 session.run(stmt)
